#!/usr/bin/env python
# python 2 syntax must be used here for gimp version <3
# -*- coding: utf-8 -*-

from gimpfu import *
import numpy as np

##############################################################################

def generate_austere(depthmap, wallpaper, d_invert, d_factor, symmetry,
                     progress_var=False, popup=False):
    h_depthmap  = depthmap.shape[0]  # height of depthmap
    w_depthmap  = depthmap.shape[1]  # width  of depthmap
    h_wallpaper = wallpaper.shape[0] # height of wallpaper
    w_wallpaper = wallpaper.shape[1] # width  of wallpaper
    idx = np.zeros(w_depthmap + w_wallpaper)
    # w_austere is a little bit larger than w_depthmap:
    w_austere = w_depthmap + int(w_wallpaper * (1-d_factor))
    austere = np.zeros((h_depthmap, w_austere, 3), dtype=np.int)
    for iy in range(h_depthmap): # loop over all lines in depthmap
        if progress_var:
            progress_var.set(int(1+100*iy/h_depthmap))
            popup.update_idletasks()
        for ix in range(w_wallpaper): # start by copying original wallpaper
            idx[ix] = ix
        d_old = w_wallpaper
        for ix in range(w_depthmap): # loop over all columns in current line
            if d_invert: # inverted depthmap?
                depthmap[iy,ix] = 1. - depthmap[iy,ix]
            d = int((1. - d_factor * depthmap[iy,ix]) * w_wallpaper)
            idx[ix+d] = idx[ix]
            if (d > d_old): # regenerate dismembered wallpaper?
                for i in range(d-d_old):
                    idx[ix+d-i-1] = (idx[ix]+w_wallpaper-i-1) % w_wallpaper
            d_old = d
        for ix in range(w_austere):
            idx_shifted = int((idx[ix] - idx[int(symmetry*w_austere)] +
                               w_wallpaper) % w_wallpaper)
            austere[iy,ix,:] = wallpaper[iy % h_wallpaper,idx_shifted,:]
    return austere

##############################################################################

def main_plugin(wallpaper, depthmap, d_invert, d_factor_percent, symmetry_percent):
    symmetry = symmetry_percent/100
    d_factor = d_factor_percent/100
    # ----- wallpaper -----
    gimp.progress_init('Reading wallpaper...')
    wallpaper_array = np.zeros((wallpaper.height, wallpaper.width, 3))
    for x in range(wallpaper.width):
        gimp.progress_update(float(x) / float(wallpaper.width)) # progress bar
        for y in range(wallpaper.height):
            wallpaper_array[y,x,:] = wallpaper.get_pixel(x,y)[0:3]
    # ----- depthmap -----
    gimp.progress_init('Reading depthmap...')
    depthmap_array = np.zeros((depthmap.height, depthmap.width))
    for x in range(depthmap.width):
        gimp.progress_update(float(x) / float(depthmap.width)) # progress bar
        for y in range(depthmap.height):
            depthmap_array[y,x] = float(depthmap.get_pixel(x,y)[0])/256.
    # ----- austere -----
    austere = generate_austere(depthmap_array, wallpaper_array, d_invert, d_factor, symmetry)
    img = gimp.Image(austere.shape[1], austere.shape[0], wallpaper.type)
    destDrawable = gimp.Layer(img, 'austere', austere.shape[1], austere.shape[0],
                              wallpaper.type, wallpaper.opacity, wallpaper.mode)
    img.add_layer(destDrawable, 0)
    gimp.progress_init('Writing austere...')
    for iy in range(austere.shape[0]):
        gimp.progress_update(float(iy) / float(austere.shape[0])) # progress bar
        for ix in range(austere.shape[1]):
            destDrawable.set_pixel(ix,iy,austere[iy,ix,:])
    gimp.Display(img)
    destDrawable.flush()

##############################################################################

register(
    'austere', # window title
    'Create AUtoSTEREograms', # header in window
    'Create AUtoSTEREograms from depthmap and wallpaper', # help text
    'Rolf Sander', # author
    'Rolf Sander', # copyright
    '2017-2020', # date
    '<Image>/Filters/Render/Autostereogram...', # menu path
    '*', # imagetypes (*, RGB, RGB*, GRAY*, INDEXED, etc.)
    [
        (PF_DRAWABLE, 'wallpaper',         'wallpaper',         None),
        (PF_DRAWABLE, 'depthmap',          'Depthmap',          None),
        (PF_BOOL,     'd_invert',          'Invert?',           False),
        (PF_SLIDER,   'd_factor_percent',  'Depth factor in %', 25, (0, 100, 1)),
        (PF_SLIDER,   'symmetry_percent',  'Symmetry in %',     50, (0, 100, 1)),
    ], # params
    [], # results
    main_plugin, # function
    # Add empty value for menu here because it is already defined above.
    # The entry cannot be omitted, otherwise "main_plugin" produces two
    # more parameters for the function call.
    menu=''
    )

main()

##############################################################################




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
assert sys.version_info >= (3, 6)
# Tkinter gui:
from tkinter import *
from tkinter.messagebox import *
from tkinter import filedialog
from tkinter import ttk # for progressbar
# other modules:
from PIL import Image, ImageTk
from austere import make_austere
import os # basename

about_text = '''Create AUtoSTEREograms
(c) Rolf Sander 2017-2020
http://www.rolf-sander.net'''

THUMBSIZE = 350
OUTPUTDIR = 'output'
WINDOWTITLE = 'AUtoSTEREogram'
INITIAL_DEPTHMAP  = 'cube.png'
INITIAL_WALLPAPER = 'birds.png'

def start_calc():
    d_factor = float(d_factor_percent.get())/100.
    symmetry = float(symmetry_percent.get())/100.
    popup = Toplevel()
    popup.title(WINDOWTITLE)
    progress_var = DoubleVar()
    progressbar = ttk.Progressbar(popup, variable=progress_var)
    progressbar.pack(padx=10, pady=10, fill=X)
    label1 = Label(popup, width=60, font='-weight bold', wraplength=500,
                   text='Please wait...')
    label1.pack(pady=10)
    popup.grab_set() # make main window inactive
    outputfile = OUTPUTDIR + '/austere_' + \
                 os.path.splitext(os.path.basename(depthmapfile))[0] + \
                 '_' + os.path.splitext(os.path.basename(wallpaperfile))[0] + '.png'
    returncode = make_austere(depthmapfile, wallpaperfile, d_invert.get(),
                              d_factor, symmetry, progress_var, popup, outputfile)
    if (returncode):
        label1['text'] = 'ERROR: %s' % (returncode)
    else:
        label1['text'] = 'The file %s has been created.' % (outputfile)
        # show austere image:
        austerepic = Button(popup, height=300, width=500)
        austerepic.pack()
        img0 = Image.open(outputfile)
        w = img0.width
        h = img0.height
        fct = min(1,float(THUMBSIZE)/max(w, h)) # scalefactor
        img0 = img0.resize((int(w*fct), int(h*fct)), Image.ANTIALIAS)
        img1 = ImageTk.PhotoImage(img0)
        austerepic.image = img1
        austerepic.config(image=img1)
    # OK button:
    okbutton = Button(popup, text='OK', font='-weight bold', command=popup.destroy)
    okbutton.pack(pady=10)
    
def show_wallpaperfile():
    img0 = Image.open(wallpaperfile)
    w = img0.width
    h = img0.height
    fct = min(1,float(THUMBSIZE)/max(w, h)) # scalefactor
    img0 = img0.resize((int(w*fct), int(h*fct)), Image.ANTIALIAS)
    img1 = ImageTk.PhotoImage(img0)
    wallpaperpic.image = img1
    wallpaperpic.config(image=img1)
    wallpaperfilelabel.config(
        font='-weight bold',
        text='Wallpaper: '+os.path.basename(wallpaperfile)+' ('+str(w)+'x'+str(h)+')')
    
def get_wallpaperfile():
    global wallpaperfile
    new_wallpaperfile = filedialog.askopenfilename(
        title = 'Select a wallpaper file',
        initialdir = 'wallpaper',
        initialfile = INITIAL_WALLPAPER,
        filetypes = [('pic files', '.png .jpg'), ('all files', '.*')])
    if (file_check(new_wallpaperfile, 3)):
        wallpaperfile = new_wallpaperfile
    else:
        popup = Toplevel()
        popup.title(WINDOWTITLE)
        Label(popup, text='Error: Choose another wallpaper file',
              fg='red', font='-weight bold').pack(padx=20, pady=20)
        Button(popup, text='OK', font='-weight bold',
               command=popup.destroy).pack(padx=20, pady=20)
        popup.grab_set() # make main window inactive
    show_wallpaperfile()
    
def show_depthmapfile():
    img0 = Image.open(depthmapfile)
    w = img0.width
    h = img0.height
    fct = min(1,float(THUMBSIZE)/max(w, h)) # scalefactor
    img0 = img0.resize((int(w*fct), int(h*fct)), Image.ANTIALIAS)
    img1 = ImageTk.PhotoImage(img0)
    depthmappic.image = img1
    depthmappic.config(image=img1)
    depthmapfilelabel.config(
        font='-weight bold',
        text='Depthmap: '+os.path.basename(depthmapfile)+' ('+str(w)+'x'+str(h)+')')

def get_depthmapfile():
    global depthmapfile
    new_depthmapfile = filedialog.askopenfilename(
        title = 'Select a depthmap file',
        initialdir = 'depthmap',
        initialfile = INITIAL_DEPTHMAP,
        filetypes = [('pic files', '.png .jpg'), ('all files', '.*')])
    if (file_check(new_depthmapfile, 1)):
        depthmapfile = new_depthmapfile
    else:
        popup = Toplevel()
        popup.title(WINDOWTITLE)
        Label(popup, text='Error: Choose another depthmap file',
              fg='red', font='-weight bold').pack(padx=20, pady=20)
        Button(popup, text='OK', font='-weight bold',
               command=popup.destroy).pack(padx=20, pady=20)
        popup.grab_set() # make main window inactive
    show_depthmapfile()

def file_check(filename, colors):
    if (not os.path.isfile(filename)):
        return False    
    # qqq
    # if (not correct colordepth(filename, colors)):
    #     return False    
    return True
    
def popupwindow_about():
    popup = Toplevel()
    popup.title(WINDOWTITLE)
    label1 = Label(popup, font='-weight bold', text=about_text)
    label1.pack(padx=20)
    Button(popup, text='OK', font='-weight bold',
           command=popup.destroy).pack(pady=20)
    popup.grab_set() # make main window inactive

gui = Tk()
gui.title(WINDOWTITLE)

wallpaperfile = 'wallpaper/' + INITIAL_WALLPAPER
wallpaperpic = Button(height=THUMBSIZE, width=THUMBSIZE, command=get_wallpaperfile)
wallpaperpic.grid(padx=20, pady=20, row=0, column=0, columnspan=2)

depthmapfile = 'depthmap/' + INITIAL_DEPTHMAP
depthmappic = Button(height=THUMBSIZE, width=THUMBSIZE, command=get_depthmapfile)
depthmappic.grid(padx=20, pady=20, row=0, column=2, columnspan=2)

wallpaperfilelabel = Label(gui)
wallpaperfilelabel.grid(row=1, column=0, columnspan=4)
show_wallpaperfile()

depthmapfilelabel = Label(gui)
depthmapfilelabel.grid(row=2, column=0, columnspan=4)
show_depthmapfile()

d_invert = BooleanVar()
Checkbutton(gui, text='Invert Depthmap?', font='-weight bold',
            variable=d_invert).grid(pady=10, row=3, column=0, columnspan=4)

d_factor_percent = Scale(gui, from_=0, to=100, orient=HORIZONTAL,
                         length=400, font='-weight bold', label='Depth Factor in %:')
d_factor_percent.grid(pady=10, row=4, column=0, columnspan=4)
d_factor_percent.set(15)

symmetry_percent = Scale(gui, from_=0, to=100, orient=HORIZONTAL,
                         length=400, font='-weight bold', label='Symmetry in %:')
symmetry_percent.grid(pady=10, row=5, column=0, columnspan=4)
symmetry_percent.set(50)

Button(gui, text='About', font='-weight bold',
       command=popupwindow_about).grid(pady=20, row=6, column=0)
Button(gui, text='Create AUtoSTEREogram', font='-weight bold',
       command=start_calc).grid(pady=20, row=6, column=1, columnspan=2)
Button(gui, text='Quit', font='-weight bold',
       #fg='red',
       command=quit).grid(pady=20, row=6, column=3)

mainloop()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__version__ =  '1.0.0'

import sys
assert sys.version_info >= (3, 6)
import numpy as np

##############################################################################

def generate_austere(depthmap, wallpaper, d_invert, d_factor, symmetry,
                     progress_var=False, popup=False):
    if (len(depthmap.shape)>2):
        return 'depthmap is probably not a greyscale image'
    if (wallpaper.shape[2]==4):
        # remove transparency layer:
        wallpaper = wallpaper[:,:,0:3]
    h_depthmap  = depthmap.shape[0]  # height of depthmap
    w_depthmap  = depthmap.shape[1]  # width  of depthmap
    h_wallpaper = wallpaper.shape[0] # height of wallpaper
    w_wallpaper = wallpaper.shape[1] # width  of wallpaper
    idx = np.zeros(w_depthmap + w_wallpaper)
    # w_austere is a little bit larger than w_depthmap:
    w_austere = w_depthmap + int(w_wallpaper * (1-d_factor))
    austere = np.zeros((h_depthmap, w_austere, 3), dtype=np.int)
    for iy in range(h_depthmap): # loop over all lines in depthmap
        if progress_var:
            progress_var.set(int(1+100*iy/h_depthmap))
            popup.update_idletasks()
        for ix in range(w_wallpaper): # start by copying original wallpaper
            idx[ix] = ix
        d_old = w_wallpaper
        for ix in range(w_depthmap): # loop over all columns in current line
            if d_invert: # inverted depthmap?
                depthmap[iy,ix] = 1. - depthmap[iy,ix]
            d = int((1. - d_factor * depthmap[iy,ix]) * w_wallpaper)
            idx[ix+d] = idx[ix]
            if (d > d_old): # regenerate dismembered wallpaper?
                for i in range(d-d_old):
                    idx[ix+d-i-1] = (idx[ix]+w_wallpaper-i-1) % w_wallpaper
            d_old = d
        for ix in range(w_austere):
            idx_shifted = int((idx[ix] - idx[int(symmetry*w_austere)] +
                               w_wallpaper) % w_wallpaper)
            austere[iy,ix,:] = wallpaper[iy % h_wallpaper,idx_shifted,:]
    return austere

##############################################################################

def make_austere(depthmapfile, wallpaperfile, d_invert, d_factor, symmetry,
                 progress_var=False, popup=False, outputfile='output/austere.png'):
    import matplotlib.pyplot as plt

    depth = plt.imread(depthmapfile)

    # imread scaled [0...255] to [0...1] so it must be rescaled back
    wallpaper = 256*plt.imread(wallpaperfile)
    
    austere = generate_austere(depth, wallpaper, d_invert, d_factor,
                               symmetry, progress_var, popup)

    if(isinstance(austere, str)):
        # an error string was returned
        return austere
    else:
        plt.imsave(outputfile, austere.astype(float)/256)    

##############################################################################

if __name__ == '__main__':

    ### DEPTHMAP ###
    #depthmapfile = 'blackhole'
    #depthmapfile = 'bumps'
    depthmapfile = 'cube'
    #depthmapfile = 'donut'
    #depthmapfile = 'ic'
    #depthmapfile = 'rings'
    #depthmapfile = 'ripples'
    #depthmapfile = 'shark'
    #depthmapfile = 'sombrero'
    #depthmapfile = 'upanddown'
    #depthmapfile = 'vortex'

    ### WALLPAPER ###
    #wallpaperfile = 'birds'
    #wallpaperfile = 'citrus'
    #wallpaperfile = 'clouds'
    #wallpaperfile = 'coffee'
    #wallpaperfile = 'flowers'
    #wallpaperfile = 'fruit'
    #wallpaperfile = 'hieroglyphs'
    #wallpaperfile = 'houses'
    #wallpaperfile = 'ink_splashes'
    wallpaperfile = 'quilt'
    #wallpaperfile = 'quilt_drops'
    #wallpaperfile = 'school'
    #wallpaperfile = 'underwater_world'
    #wallpaperfile = 'water'

    ### OPTIONS ###
    d_invert = False # invert up and down
    d_factor = 0.25  # 0...1
    symmetry = 0.5   # 0...1

    print(f'\nCreating an autostereogram from the depthmap "{depthmapfile}"')
    print(f'and the wallpaper "{wallpaperfile}"...\n')

    returncode = make_austere('depthmap/'+depthmapfile+'.png',
                              'wallpaper/'+wallpaperfile+'.png',
                              d_invert,
                              d_factor,
                              symmetry)
    if (returncode):
        print(f'ERROR: {returncode}')
    else:
        print('The image file "output/austere.png" has been created.\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import numpy as np
import sys
import scipy.misc
from math import atan, cos, exp, pi, sin, sqrt

depthxSize = 1400; depthySize = 1050 # 4:3 aspect ratio
wallpaperxSize = 100 # horizontal shift of depth image

##############################################################################

def make_depthmap(function):

    NColors = 256
    print function
    depthmap_picture = np.zeros((depthySize,depthxSize))

    for irow in range(depthySize):
        for icol in range(depthxSize):
            # x,y are roughly in the range -1, ..., +1:
            x = 1.1 * (2.*icol+0.5*wallpaperxSize-depthxSize) / depthySize
            y = 1.1 * (depthySize-2.*irow)                    / depthySize
            r2 = x**2 + y**2
            if (function == 'blackhole'):
                depth = (NColors-1 - int((NColors-1) * exp(-2*r2) + 0.5))
            if (function == 'bumps'):
                depth = sin(5*x)*cos(5*y)
            if (function == 'donut'):
                inner = 0.4
                outer = 0.95
                if r2 < outer**2 and r2 > inner**2:
                    tmp = (2*sqrt(r2)-inner-outer) / (outer-inner)
                    depth = (NColors/2 + int((NColors/2-1) * sqrt(1-tmp**2) + 0.5))
                else:
                    depth = 0
            if (function == 'ripples'):
                depth = (NColors/2) + (NColors/2) * cos(10*sqrt(r2)) * max(1-0.3*r2,0)
            if (function == 'sombrero'):
                depth = NColors * abs(1.5*r2-1)
                if (r2>1.7): depth = 0
                if ((1.5*r2>1) and (depth>0.4*NColors)):
                    depth = 0.4*NColors
            if (function == 'upanddown'):
                depth = NColors * (x + y) * exp(-4.*r2)
                if (depth<-NColors): depth = -NColors
                if (depth>NColors):  depth = NColors
            if (function == 'vortex'):
                if (x==0):
                    angle = 3*pi/2 if (y<0) else pi/2
                else:
                    angle = atan(y / x)
                if (x<0):
                    angle += pi
                gauss = exp(-2*r2)
                depth = int(0.5 + (NColors-1) * (1 - gauss**2) *
                    (1 + 0.5 * gauss * (sin(5 * (3*r2*pi/4-angle)))) - 1)
            depthmap_picture[irow,icol] = depth

    # produce grayscale image:
    scipy.misc.imsave('depthmap/' + function + '.png', depthmap_picture)

##############################################################################

if __name__ == '__main__':

    make_depthmap('blackhole')
    make_depthmap('bumps')
    make_depthmap('donut')
    make_depthmap('ripples')
    make_depthmap('sombrero')    
    make_depthmap('upanddown')
    make_depthmap('vortex')

##############################################################################



#!/usr/bin/env python3
# -*- coding: utf-8 -*- Time-stamp: <2021-04-07 11:40:15 sander>*-

# - tutorial:
#   https://packaging.python.org/tutorials/packaging-projects
# - full documentation:
#   https://packaging.python.org/guides/distributing-packages-using-setuptools
# - more hints:
#   http://shallowsky.com/blog/programming/developing-pypi-project.html
#   https://realpython.com/pypi-publish-python-package

# ----------------------------------------------------------------------------

import setuptools
import re
import sys

# ----------------------------------------------------------------------------

# - write README.md in GitHub Flavored Markdown language:
#   https://gist.github.com/stevenyap/7038119
#   https://jbt.github.io/markdown-editor
#   https://github.github.com/gfm
with open('README.md', 'r') as f:
    long_description = f.read()

# ----------------------------------------------------------------------------

# get version number from austere.py:
# https://packaging.python.org/guides/single-sourcing-package-version
def get_version():
    with open('austere/austere.py', 'r') as f:
        for line in f:
            result = re.search('^\s*__version__\s*=\s*[\'"]([0-9.]+)[\'"]', line)
            if (result):
                return result.group(1)
    sys.exit('ERROR: No version number found.')

# ----------------------------------------------------------------------------

setuptools.setup(
    name                          = 'austere3d',
    version                       = get_version(),
    author                        = 'Rolf Sander',
    author_email                  = 'mail@rolf-sander.net',
    description                   = 'Create your own AUtoSTEREograms',
    long_description              = long_description,
    long_description_content_type = 'text/markdown',
    url                           = 'https://gitlab.com/RolfSander/austere',
    packages                      = setuptools.find_packages(),
    install_requires              = ['matplotlib', 'numpy', 'pillow', 'scipy'],
    include_package_data          = True,
    classifiers                   = [ # https://pypi.org/classifiers
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Topic :: Multimedia :: Graphics',
        ],
    python_requires               = '>=3.6',
)

# ----------------------------------------------------------------------------
